MAIN_ENV
#include "stdio.h"
#include "comments.h"	/* moves leading comments to another file */
#include "split.h"

/*  include files for declarations  */
#define extern
#include "parameters.h"
#include "mdvar.h"
#include "water.h"
#include "wwpot.h"
#include "cnst.h"
#include "mddata.h"
#include "fileio.h"
#include "frcnst.h"
#include "randno.h"
#include "global.h"
#undef extern

struct GlobalMemory *gl;        /* pointer to the Global Memory
                                structure, which contains the lock,
                                barrier, and some scalar variables */
                                
int phase = 0;
int NSTEP, NSAVE, NRST, NPRINT,NFMC;
int NORD1;
int II;                         /*  variables explained in common.h */
int i;
int NDATA;
int   NFRST=11;
int  NFSV=10;
int  LKT=0;

int ProcID;                     /* process id; one per process */
int StartMol[MAXPROCS+1];       /* number of the first molecule
                                   to be handled by this process; used
                                   for static scheduling     */ 
int MolsPerProc;                /* number of mols per processor */ 
int NumProcs;                   /* number of processors being used; 
                                   run-time input           */
void WorkStart();

char file1[128];
char file2[128];
char file3[128];
char suffix[128];

main(argc, argv)
char **argv;
{
    int mol, func, dir, atom;
    double XTT, MDMAIN();
    double VIR;
    int extra;

    /* default values for the control parameters of the driver */
    /* are in parameters.h */

    six = fopen("LWO6","w");  /* output file */
    TEMP  =298.0;
    RHO   =0.9980;
    CUTOFF=0.0;

   /* READ INPUT */

    /*
     *   TSTEP = time interval between steps
     *   NMOL  = number of molecules to be simulated
     *   NSTEP = number of time-steps to be simulated
     *   NORDER = order of the predictor-corrector method. 6 by default
     *   NSAVE = frequency of data saving.  -1 by default
     *   NRST  = no longer used
     *   NPRINT = frequency (in time-steps) of computing pot. energy and
     *            writing output file.  setting it larger than NSTEP
     *            means not doing any I/O until the very end.
     *   NFMC = file number to read initial displacements from.  set to 
     *          0 if program should generate a regular lattice initially.
     *   NumProcs = number of processors to be used.
     */
            
    scanf("%lf%d%d%d%d%d%d%d%d",&TSTEP, &NMOL, &NSTEP, &NORDER, 
	   &NSAVE, &NRST, &NPRINT, &NFMC,&NumProcs);
    if (NMOL > MAXMOLS) {
	fprintf(stderr, "Lock array in global.H has size %d < %d (NMOL)\n",
							MAXMOLS, NMOL);
	exit(-1);
    }

    printf("Using %d procs on %d steps of %d mols\n", NumProcs, NSTEP, NMOL);

    /* SET UP SCALING FACTORS AND CONSTANTS */

    NORD1=NORDER+1;

    CNSTNT(NORD1,TLC);  /* sub. call to set up constants */

    fprintf(six,"\nTEMPERATURE                = %8.2f K\n",TEMP);
    fprintf(six,"DENSITY                    = %8.5f G/C.C.\n",RHO);
    fprintf(six,"NUMBER OF MOLECULES        = %8d\n",NMOL);
    fprintf(six,"NUMBER OF PROCESSORS       = %8d\n",NumProcs);
    fprintf(six,"TIME STEP                  = %8.2e SEC\n",TSTEP);
    fprintf(six,"ORDER USED TO SOLVE F=MA   = %8d \n",NORDER);
    fprintf(six,"NO. OF TIME STEPS          = %8d \n",NSTEP);
    fprintf(six,"FREQUENCY OF DATA SAVING   = %8d \n",NSAVE);
    fprintf(six,"FREQUENCY TO WRITE RST FILE= %8d \n",NRST);

    { /* Do memory initializations */
	int pid;
	unsigned mol_size = sizeof(molecule_type) * NMOL;
	unsigned gmem_size = sizeof(struct GlobalMemory);
	MAIN_INITENV();  /* macro call to initialize
                            shared memory etc. */
      /* allocate space for main (VAR) data structure as well as
           synchronization variables */
	VAR = (molecule_type *) G_MALLOC(mol_size);
	gl = (struct GlobalMemory *) G_MALLOC(gmem_size);
        gl->initialized = 0;

             /* macro calls to initialize synch varibles  */

	BARINIT(gl->start)
	BARINIT(gl->InterfBar)
	BARINIT(gl->PotengBar)
        LOCKINIT(gl->IntrafVirLock)
        LOCKINIT(gl->InterfVirLock)
        LOCKINIT(gl->FXLock)
        LOCKINIT(gl->FYLock)
        LOCKINIT(gl->FZLock)
	ALOCKINIT(gl->MolLock, NMOL)
	LOCKINIT(gl->KinetiSumLock)
	LOCKINIT(gl->PotengSumLock)

            /* set up control for static scheduling */

	MolsPerProc = NMOL/NumProcs;
	extra = NMOL % NumProcs;
	StartMol[0] = 0;
	for (pid = 1; pid < NumProcs; pid += 1) {
          StartMol[pid] = StartMol[pid-1] + MolsPerProc;
	  if (pid <= extra) StartMol[pid]++;
        }
	StartMol[NumProcs] = NMOL;
      }

    /* association of addresses to nodes */
    {
      int pid;
    
      for (pid = 0; pid<NumProcs; pid++)
	{
	  AssociateAddrNode((void *)&VAR[StartMol[pid]], (void *)&VAR[StartMol[pid+1]],
			    pid, "Molecules");
	  AssociateAddrNode((void *)&(gl->MolLock[StartMol[pid]]),
			    (void *)&(gl->MolLock[StartMol[pid+1]]),
			    pid, "MolLocks");
	}

      AssociateAddrNode((void *)gl,(void *)&(gl->MolLock[0]), 0, "Global");
      AssociateAddrNode((void *)(&(gl->start)),(void *)(&gl->initialized + 1), 0, "Global");
    }
      
    SYSCNS();    /* sub. call to initialize system constants  */

    fprintf(six,"SPHERICAL CUTOFF RADIUS    = %8.4f ANGSTROM\n",CUTOFF);
    fflush(six);

    IRST=0;

            /* if there is no input displacement file, and we are to
               initialize to a regular lattice */
    if (NFMC != 0) nfmc = fopen("LWI12","r"); /* input file for particle
                                  displacements */
    else nfmc = NULL;


            /* initialization routine; also reads displacements and
             sets up random velocities*/
    INITIA(nfmc);
#ifdef DEBUG
    printf("Proc %d right after INITIA \n",ProcID); fflush(stdout);
#endif
    /*.......ESTIMATE ACCELERATION FROM F/M */
    {       
            /* note that these initial calls to the force-computing 
               routines  (INTRAF and INTERF) use only 1 process since 
               others haven't been created yet */
    int tmp_procs = NumProcs;
    int tmp_smol = StartMol[1];
    ProcID = 0;
    StartMol[0] = 0;
    StartMol[1] = NMOL;
    NumProcs = 1;
    /*    INTRAF(&VIR);
    INTERF(ACC,&VIR); */
    
    StartMol[1] = tmp_smol;
    NumProcs = tmp_procs;
    }  /* Acclerations estimated */


    {
      unsigned mol_size = sizeof(molecule_type) * NMOL;

      sprintf(suffix,"%d",NMOL);
      printf("suffix = %s\n", suffix);
      strcpy(file2,"VAR" );
      strcpy(file3,"VIR");
      strcat(file2, suffix);
      strcat(file3, suffix);
      
      SpecialInitInput((char *)VAR, mol_size , file2);
      SpecialInitInput((char *)&VIR,sizeof(double) , file3);
    }


    NFMC= -1;

    /*.....START MOLECULAR DYNAMIC LOOP */
    if (NFMC < 0) {
  	ELPST=0.00;
        TKIN=0.00;
        TVIR=0.00;
        TTMV=0.00;
    };

    MEMSYS_ON /* turns on memory simulation */

#ifdef DEBUG
    printf("Proc %d spawning the slave processes\n",ProcID); fflush(stdout);
#endif
      
       /* spawn helper processes, each getting its unique process id */
    for (ProcID = 1; ProcID < NumProcs; ProcID += 1) {
	CREATE(WorkStart);
    }

    ProcID = 0; /* process id of main process */

    if (NSAVE > 0)  /* not true for input decks provided */
      fprintf(six,"COLLECTING X AND V DATA AT EVERY %4d TIME STEPS \n",NSAVE);

#ifdef DEBUG
    printf("Proc %d about to enter MDMAIN \n",ProcID); fflush(stdout);
#endif
            /* call routine to do the timesteps */
    XTT = MDMAIN(NFSV,NFRST,NSTEP,NRST,NPRINT,NSAVE,LKT,NORD1); 

            /* macro to make main process wait for all others to finish */
    WAIT_FOR_END(NumProcs-1)

            /* signal completion to verification output file LWV */
    one = fopen("LWV","w");
    fprintf(one, "\nExited Happily with XTT %g\n", XTT);

    MAIN_END
} /* main.c */

void WorkStart() /* routine that each created process starts at;
              it simply calls the timestep routine */
{
  double MDMAIN();
#ifdef DEBUG
  printf("Proc %d calling MDMAIN with NumProcs = %d at %d\n",
	 ProcID, NumProcs, time(0)); fflush(stdout);
#endif
    (void) MDMAIN(NFSV,NFRST,NSTEP,NRST,NPRINT,NSAVE,LKT,NORD1);
}
